import os

import pooch

from . import BiRefNetSessionGeneral


class BiRefNetSessionDIS(BiRefNetSessionGeneral):
    """
    This class represents a BiRefNet-DIS session, which is a subclass of BiRefNetSessionGeneral.
    """

    @classmethod
    def download_models(cls, *args, **kwargs):
        """
        Downloads the BiRefNet-DIS model file from a specific URL and saves it.

        Parameters:
            *args: Additional positional arguments.
            **kwargs: Additional keyword arguments.

        Returns:
            str: The path to the downloaded model file.
        """
        fname = f"{cls.name(*args, **kwargs)}.onnx"
        pooch.retrieve(
            "https://github.com/danielgatis/rembg/releases/download/v0.0.0/BiRefNet-DIS-epoch_590.onnx",
            (
                None
                if cls.checksum_disabled(*args, **kwargs)
                else "md5:2d4d44102b446f33a4ebb2e56c051f2b"
            ),
            fname=fname,
            path=cls.u2net_home(*args, **kwargs),
            progressbar=True,
        )

        return os.path.join(cls.u2net_home(*args, **kwargs), fname)

    @classmethod
    def name(cls, *args, **kwargs):
        """
        Returns the name of the BiRefNet-DIS session.

        Parameters:
            *args: Additional positional arguments.
            **kwargs: Additional keyword arguments.

        Returns:
            str: The name of the session.
        """
        return "birefnet-dis"



import click

from ..bg import download_models


@click.command(  # type: ignore
    name="d",
    help="download models",
)
@click.argument("models", nargs=-1)
def d_command(models: tuple[str, ...]) -> None:
    """
    Download models
    """
    download_models(models)



import os
from typing import List

import numpy as np
import onnxruntime as ort
from PIL import Image
from PIL.Image import Image as PILImage

from .base import BaseSession


class BenCustomSession(BaseSession):
    """This is a class representing a custom session for the Ben model."""

    def __init__(self, model_name: str, sess_opts: ort.SessionOptions, *args, **kwargs):
        """
        Initialize a new BenCustomSession object.

        Parameters:
            model_name (str): The name of the model.
            sess_opts: The session options.
            *args: Additional positional arguments.
            **kwargs: Additional keyword arguments.
        """
        model_path = kwargs.get("model_path")
        if model_path is None:
            raise ValueError("model_path is required")

        super().__init__(model_name, sess_opts, *args, **kwargs)

    def predict(self, img: PILImage, *args, **kwargs) -> List[PILImage]:
        """
        Predicts the mask image for the input image.

        This method takes a PILImage object as input and returns a list of PILImage objects as output. It performs several image processing operations to generate the mask image.

        Parameters:
            img (PILImage): The input image.

        Returns:
            List[PILImage]: A list of PILImage objects representing the generated mask image.
        """

        ort_outs = self.inner_session.run(
            None,
            self.normalize(img, (0.5, 0.5, 0.5), (1.0, 1.0, 1.0), (1024, 1024)),
        )

        pred = ort_outs[0][:, 0, :, :]

        ma = np.max(pred)
        mi = np.min(pred)

        pred = (pred - mi) / (ma - mi)
        pred = np.squeeze(pred)

        mask = Image.fromarray((pred * 255).astype("uint8"), mode="L")
        mask = mask.resize(img.size, Image.Resampling.LANCZOS)

        return [mask]

    @classmethod
    def download_models(cls, *args, **kwargs):
        """
        Download the model files.

        Parameters:
            *args: Additional positional arguments.
            **kwargs: Additional keyword arguments.

        Returns:
            str: The absolute path to the model files.
        """
        model_path = kwargs.get("model_path")
        if model_path is None:
            raise ValueError("model_path is required")

        return os.path.abspath(os.path.expanduser(model_path))

    @classmethod
    def name(cls, *args, **kwargs):
        """
        Get the name of the model.

        Parameters:
            *args: Additional positional arguments.
            **kwargs: Additional keyword arguments.

        Returns:
            str: The name of the model.
        """
        return "ben_custom"



try:
    from importlib.metadata import PackageNotFoundError, version

    try:
        __version__ = version("rembg")
    except PackageNotFoundError:
        __version__ = "0.0.0"  # Fallback for development
except ImportError:
    __version__ = "0.0.0"  # Fallback for older Python versions

from .bg import remove
from .session_factory import new_session



from __future__ import annotations

from typing import Dict, List

from .base import BaseSession

sessions: Dict[str, type[BaseSession]] = {}

from .birefnet_general import BiRefNetSessionGeneral

sessions[BiRefNetSessionGeneral.name()] = BiRefNetSessionGeneral

from .birefnet_general_lite import BiRefNetSessionGeneralLite

sessions[BiRefNetSessionGeneralLite.name()] = BiRefNetSessionGeneralLite

from .birefnet_portrait import BiRefNetSessionPortrait

sessions[BiRefNetSessionPortrait.name()] = BiRefNetSessionPortrait

from .birefnet_dis import BiRefNetSessionDIS

sessions[BiRefNetSessionDIS.name()] = BiRefNetSessionDIS

from .birefnet_hrsod import BiRefNetSessionHRSOD

sessions[BiRefNetSessionHRSOD.name()] = BiRefNetSessionHRSOD

from .birefnet_cod import BiRefNetSessionCOD

sessions[BiRefNetSessionCOD.name()] = BiRefNetSessionCOD

from .birefnet_massive import BiRefNetSessionMassive

sessions[BiRefNetSessionMassive.name()] = BiRefNetSessionMassive

from .dis_anime import DisSession

sessions[DisSession.name()] = DisSession

from .dis_custom import DisCustomSession

sessions[DisCustomSession.name()] = DisCustomSession

from .dis_general_use import DisSession as DisSessionGeneralUse

sessions[DisSessionGeneralUse.name()] = DisSessionGeneralUse

from .sam import SamSession

sessions[SamSession.name()] = SamSession

from .silueta import SiluetaSession

sessions[SiluetaSession.name()] = SiluetaSession

from .u2net_cloth_seg import Unet2ClothSession

sessions[Unet2ClothSession.name()] = Unet2ClothSession

from .u2net_custom import U2netCustomSession

sessions[U2netCustomSession.name()] = U2netCustomSession

from .u2net_human_seg import U2netHumanSegSession

sessions[U2netHumanSegSession.name()] = U2netHumanSegSession

from .u2net import U2netSession

sessions[U2netSession.name()] = U2netSession

from .u2netp import U2netpSession

sessions[U2netpSession.name()] = U2netpSession

from .bria_rmbg import BriaRmBgSession

sessions[BriaRmBgSession.name()] = BriaRmBgSession

from .ben_custom import BenCustomSession

sessions[BenCustomSession.name()] = BenCustomSession

sessions_names = list(sessions.keys())
sessions_class = list(sessions.values())



import sys

# Fast path for --version (avoid importing heavy dependencies)
if len(sys.argv) == 2 and sys.argv[1] in ("--version", "-V"):
    from importlib.metadata import version

    print(f"rembg, version {version('rembg')}")
    sys.exit(0)

try:
    import click
except ImportError:
    print("The CLI dependencies are not installed.")
    print("Please install rembg with CLI support:")
    print()
    print('    pip install "rembg[cpu,cli]"  # for CPU')
    print('    pip install "rembg[gpu,cli]"  # for NVIDIA/CUDA GPU')
    print()
    print(
        "For more information, see: https://github.com/danielgatis/rembg#installation"
    )
    sys.exit(1)

from . import __version__
from .commands import command_functions


@click.group()
@click.version_option(version=__version__)
def main() -> None:
    pass


for command in command_functions:
    main.add_command(command)
